;A little puzzle game...

#include "ti82.h"
#include "keys.inc"

MAXLEVEL = 100

.org START_ADDR
.db "Picross v1.3",0

Level           = TEXT_MEM
Timer           = TEXT_MEM+1
t_delay         = TEXT_MEM+2
Size            = TEXT_MEM2
Score           = TEXT_MEM+5
SprY            = TEXT_MEM+8
SprX            = TEXT_MEM+9
Live            = TEXT_MEM+10
InfoTxtH        = TEXT_MEM+12
InfoTxtV        = TEXT_MEM+42
string          = TEXT_MEM+72
	 set 7,(IY+$14)
	 call ClearGraphMem

prog_start:

	 LD A,$8C           ;Needed for
	 OUT (2),A          ;FIND_PIXEL
	 ld       (iy+13),0 ;No scrolling text!
Start:
	sub     A
	ld      (Level), A
	ROM_CALL(CLEARLCD)
	ld      A, (BestLevel)
	and     A
	jr      z, NoLevelSelect
	ld      HL, LevelSelectStr
	ld      de, $0100
      call    DTX
      ld      hl,lev
      ld      d,8
      call    DTX
	ld      B, 0
	jr      RedrawLevelSel
LevelSelect:
      ei
	halt
	call    GET_KEY
	and     A
	jr      z, LevelSelect
	cp      G_UP
	jr      z, NextLevel
	cp      G_DOWN
	jr      z, PreviousLevel
	cp      G_ENTER
	jr      z, LevelSelectDone
	cp      G_CLEAR
	jr      nz, LevelSelect
ClearBestLevel:
	sub     A
	ld      (BestLevel), A
	jr      NoLevelSelect
NextLevel:
	ld      A, (BestLevel)
	cp      B
	jr      z, LevelSelect
	inc     B
	jr      RedrawLevelSel
PreviousLevel:
	inc     B
	dec     B
	jr      z, LevelSelect
	dec     B
RedrawLevelSel:
	ld      de, $0F03
	ld      L, B
	inc     L
	ld      H, 0
	call    DCX
	jr      LevelSelect
LevelSelectDone:
	ld      A, B
	ld      (Level), A
NoLevelSelect:
	ld      A, 3
	ld      (Live), A
      ld      a,100
      ld      (t_delay),a
	ld      HL, 0
	ld      (Score), HL
GameStart:
	ROM_CALL(CLEARLCD)
	call    DrawStuff
	call    PrepareLevel
	ld      BC, 0
	call    DrawPointer
	call    GenHorzInfo
	call    GenVertInfo
GameLoop:
GameLoop_:
 ei
	halt
      call    DoTimer
	call    GET_KEY
	and     A
	jr      z, GameLoop
	push    AF
	call    DrawPointer
	call    DispHorzInfo
	call    DispVertInfo
	ld      A, (Size)
	and     A
	ld      D, 7
	jr      z, MaxSize
	ld      D, 15
MaxSize:
	pop     AF
	cp      G_UP
	jr      nz, CheckDown
	ld      A, C
	and     A
	jr     z, RedrawPointer
	dec     C
	jr     RedrawPointer
CheckDown:
	cp      G_DOWN
	jr      nz, CheckLeft
	ld      A, C
	cp      D
	jr      nc, RedrawPointer
	inc     C
	jr      RedrawPointer
CheckLeft:
	cp      G_LEFT
	jr      nz, CheckRight
	ld      A, B
	and     A
	jr      z, RedrawPointer
	dec     B
	jr      RedrawPointer
CheckRight:
	cp      G_RIGTH
	jr      nz, CheckSecond
	ld      A, B
	cp      D
	jr      nc, RedrawPointer
	inc     B
	jr      RedrawPointer
CheckSecond:
	cp      G_2nd
	jr      nz, CheckPause
	call    GenMapOffset
	ld      HL, UserMatrix
	add     HL, DE
	push    HL
	ld      A, (HL)
	and     A
	jr      nz, CheckSecond_
	ld      HL, LevelMatrix
	add     HL, DE
	ld      A, (HL)
	and     A
	jr      nz, CheckSecond_
	pop     HL
	ld      A, (Timer)
	sub     5
	ld      (Timer), A
	push af
	call    DispMiss
	pop af
	jr      c, LoseLife
      jr      z, LoseLife
	jr      RedrawPointer
CheckSecond_:
	pop     HL
	ld      A, (HL)
	xor     1
	ld      (HL), A
	call    FillTile
	call    Compare
	jr      RedrawPointer
CheckPause:
	cp      G_MODE
	jr      nz, CheckExit
	call    DisplaySomeInfo
	call    PAUSE
	jr      RedrawPointer
CheckExit:
	cp      G_CLEAR
	jr      z, Exit
RedrawPointer:
	call    GenHorzInfo
	call    GenVertInfo
	call    DrawPointer
	call    DispHorzInfo
	call    DispVertInfo
	jp      GameLoop

LoseLife:
	set   3,(IY+5)
	ld      HL, TimeUpStr
	ld      de, $0303
	call    DTX
	res   3,(IY+5)
	call    PAUSE
	call    ClearGraphMem
	ld      hl,Live
	dec (hl)
	jp      nz, GameStart
GameIsOver:
      set     3,(iy+5)
	ld      HL, GameOverStr
	ld      de,0
	call DTX
      res     3,(iy+5)
	call    PAUSE
Exit:
ClearGraphMem:
 ld hl,GRAPH_MEM
 ld bc,767
 jp CLEAR

PAUSE:
 call GET_KEY
 cp G_ENTER
 jr nz,PAUSE
 ret


GameDone:
	ROM_CALL(CLEARLCD)
	ld      HL, DoneStr
	ld      de, 0
	call    DTX
	call    PAUSE
	jr      GameIsOver

DispMiss:
      set     3,(iy+5)
	ld      HL, MissStr
	ld      de, $0504
	call    DTX
      res     3,(iy+5)
      call Delay
      call Delay
      call Delay
      jp Delay

DrawStuff:
	set 7, (iy+$14)
      ld      DE, (0*256)+10
      ld hl,Title
	call DMX
	ld      de, (6*256)+16
      call DMX
	ld      DE, (12*256)+18
	call DMX
	ld      DE, (18*256)+9
	call DMX
	ld      DE, (24*256)+9
	call DMX
	res 7, (iy+$14)
	ld      BC, 31
	ld      D, 96
      call    HLine

	ld      BC, 65*256
	ld      D, 64
	call    VLine

      ld      BC, 2*256
      ld      D, 65
      jp      VLine

;horizontal line: B = start x, D = len, C = y
HLine:
	ld      A, 63
	sub     C
	ld      C, A
	call    FIND_PIXEL
	ld      B, D
	ld      DE, GRAPH_MEM+1
	add     HL, DE
	ld      C, A
HLineLoop:
	ld      A, (HL)
	or      C
	ld      (HL), A
	rrc     C
	jr      nc, HLineLoop_
	inc     HL
HLineLoop_:
	djnz    HLineLoop
	ret

;vertical line: B = x, C = start y, D = len
VLine:
	ld      A, 63
	sub     C
	ld      C, A
	call    FIND_PIXEL
	ld      B, D
	ld      DE, GRAPH_MEM
	add     HL, DE
	ld      C, A
	ld      DE, 12
VLineLoop:
	ld      A, (HL)
	or      C
	ld      (HL), A
	add     HL, DE
	djnz    VLineLoop
	ret

;xor tile (B=x,C=y)
FillTile:
	push    BC
	call    CalcPos
	ld      A, (Size)
	ld      HL, Tile44
	and     A
	jr      z, FillTile_
	ld      HL, Tile22
FillTile_:
	call    PutSprite
	pop     BC
	ret

;xor tile (B=x,C=y)
DrawPointer:
	push    BC
	call    CalcPos
	ld      HL, Pointer
	call    PutSprite
	pop     BC
	ret  

CalcPos:
	sla     B
	sla     C
	ld      A, (Size)
	and     A
	jr      nz, CalcPos_
	sla     B
	sla     C
CalcPos_:
	ld      A, B
	add     A, 66
	ld      B, A
	ld      A, C
	add     A, 32
	ld      C, A
	ret

PrepareLevel:
	ld      A, (Level)
	ld      B, A
	ld      HL, Levels
	and     A
	jr      z, GotLevel
	ld      DE, 8+2
GetLevel:
	add     HL, DE
	ld      A, (HL)
	and     A
	ld      E, 8+2
	jr      z, GetLevel_
	ld      E, 32+2
GetLevel_:
	djnz    GetLevel
GotLevel:
	ld      A, (HL)
	ld      (Size), A
	inc     HL
	ld      A, (HL)
	ld      (Timer), A

	inc     HL
;"uncompress" level
	ld      B, 128
	ld      A, (Size)
	and     A
	ld      C, 8
	jr      z, UncompressLevel_
	ld      C, 32
UncompressLevel_:
	ld      DE, LevelMatrix
UncompressLevel:
	ld      A, (HL)
	and     B
	ld      A, 0
	jr      z, SetMatrix
	inc     A
SetMatrix:
	ld      (DE), A
	inc     DE
	rrc     B
	jr      nc, UncompressLevel
	inc     HL
	dec     C
	jr      nz, UncompressLevel
      push    bc
	call DispTime
      set 7, (iy+$14)
	ld      DE, (24*256)+32
	ld      (CURSOR_X), DE
	ld      A, (Live)
	ld      L, A
	ld      H, 0
      ld      b,1
	call    DM_HL_DECI
	res 7, (iy+$14)
      pop bc

	ld      HL, UserMatrix
	ld      BC, 255
CLEAR:
 ld d,h
 ld e,l
 inc de
 ld (hl),0
 ldir
 ret

GenMapOffset:
	push    BC
	sla     C
	sla     C
	sla     C
	ld      A, (Size)
	and     A
	jr      z, GenMapOffset_
	sla     C
GenMapOffset_:
	ld      A, C
	add     A, B
	ld      E, A
	ld      D, 0
	pop     BC
	ret

Compare:
	push    BC
	ld      A, (Size)
	ld      B, 64
	and     A
	jr      z, Compare_
	ld      B, 0
Compare_:
	ld      HL, UserMatrix
	ld      DE, LevelMatrix
CompareLoop:
	ld      A, (DE)
	cp      (HL)
	jr      nz, NotSame
	inc     HL
	inc     DE
	djnz    CompareLoop
Same:
;count down timer, up score, set best score, next level
ScoreLoop:
      call    Delay
	ld      hl, Timer
      dec     (hl)
	jr      z, ScoreLoopDone
	ld      HL, (Score)
	inc     HL
      ld      (Score),hl
	ld      DE, (BestScore)
	ld      HL, (Score)
	call    CP_HL_DE
	jr      c, ScoreLoop
      ld      (BestScore),HL
	jr      ScoreLoop
ScoreLoopDone:
      set     3,(iy+5)
	ld      HL, StageDone
	ld      de, $0200
	call    DTX
      res     3,(iy+5)
	call    PAUSE
	call    ClearGraphMem
	ld      A, (BestLevel)
	ld      B, A
	ld      A, (Level)
	inc     A
	ld      (Level), A
	cp      B
	jr      c, NoBestLevel
	ld      (BestLevel), A
NoBestLevel:
	cp      MAXLEVEL
	pop     BC
	pop     HL
	jp      c, GameStart
	jp      GameDone
NotSame:
	pop     BC
	ret


DispTime:
      set 7,(iy+$14)
	ld      DE, (18*256)+32
	ld      (CURSOR_X), DE
      ld  a,(Timer)
      ld  l,a
      ld  h,0
      ld      b,3
	call    DM_HL_DECI
      res 7,(iy+$14)

	ld      DE, (18*256)+24
	ld      (CURSOR_X), DE
      ld  a,(Timer)
      ld  l,a
      ld  h,0
      ld      b,3
	call    DM_HL_DECI
	ret

;blit 4x4 sprite (HL) to B,C
PutSprite:
	push    BC
	push    DE
	ld      DE, GRAPH_MEM
convdone:
	ld      A, 63
	sub     C
	ld      C, A
	push    HL
	push    HL
	call    FIND_PIXEL
	add     HL, DE
	ex      DE, HL
	pop     HL
	ld      B, (HL)
	inc     HL
	ld      C, (HL)
	inc     HL
	push    HL
	pop     IX
	ex      DE, HL
PS_NewRow:
	push    BC
	ld      D, (IX)
	inc     IX
	push    AF
	push    HL
PS_NewCol:
	rl      D
	ld      E, A
	jr      nc, PS_NextPixel
	xor     (HL)
	ld      (HL), A
PS_NextPixel:
	ld      A, E
	rrca
	jr      nc, PS_SameByte
	inc     HL
PS_SameByte:
	djnz    PS_NewCol
	pop     HL
	pop     AF
	ld      DE, 12
	add     HL, DE
	pop     BC
	dec     C
	jr      nz, PS_NewRow
	pop     HL
	pop     DE
	pop     BC
	jp SHOW_GRAPH


;C = horizontal number
GenHorzInfo:
        push    BC
        sla     C
        sla     C
        sla     C
        ld      A, (Size)
        and     A
        jr      z, GenHorzInfo_
        sla     C
GenHorzInfo_:
        ld      HL, LevelMatrix
        ld      B, 0
        add     HL, BC
        ld      DE, InfoTxtH
        push    DE
        ld      DE, 0
SearchBlackX_Loop:
        ld      A, (Size)
        and     A
        ld      B, 8
        jr      z, SearchBlackX
        ld      B, 16   ;12		;;;;;;;
SearchBlackX:
        ld      A, (HL)
        and     A
        jr      nz, SearchBlackX_1
SearchBlackX_0:
        cp      E                       ;last tile?
        jr      z, SearchBlackX_
DispNum_X:
        ld      A, D
        pop     DE
        cp      10
        jr      c, DispNum_X_
        add     A, 'A'-'0'-10
DispNum_X_:
        add     A, '0'
        ld      (DE), A
        inc     DE
        ld      A, ' '
        ld      (DE), A
        inc     DE
        push    DE
        ld      D, 0
        jr      SearchBlackX_
SearchBlackX_1:
        inc     D
        ld	ixh,b		;.db     $DD \ ld H, B
        dec	ixh		;.db     $DD \ dec H
        jr      z, DispNum_X
        cp      E
        jr      z, SearchBlackX_
        ld      D, 1
SearchBlackX_:
        ld      E, (HL)
        inc     HL
        djnz    SearchBlackX
        pop     DE
        sub     A
        ld      (DE), A
        pop     BC
        call    DispHorzInfo
        ret

;C = horizontal number
DispHorzInfo:
        push    BC
        sla     C
        ld      A, (Size)
        and     A
        jr      nz, DispHorzInfo_1
        sla     C
DispHorzInfo_1:
        ld      A, C
        add     A, 31
        cp      58
        jr      c, DispHorzInfo_
        ld      A, 58
DispHorzInfo_:
	ld	h,a
	ld	l,1
;	ld      (CURSOR_Y), A		;;optimizable
;	ld      A, 1
;	ld      (CURSOR_X), A
	ld      (CURSOR_X), hl
        ld      HL, InfoTxtH
        ROM_CALL(D_ZM_STR)
        pop     BC
        ret

;B = vertical number
GenVertInfo:
        push    BC
        ld      HL, LevelMatrix
        ld      C, B
        ld      B, 0
        add     HL, BC
        ld      DE, InfoTxtV
        push    DE
        ld      DE, 0
SearchBlackY_Loop:
        ld      A, (Size)
        and     A
        ld      B, 8
        jr      z, SearchBlackY
        ld      B, 16 ;12
SearchBlackY:
        ld      A, (HL)
        and     A
        jr      nz, SearchBlackY_1
SearchBlackY_0:
        cp      E                       ;last tile?
        jr      z, SearchBlackY_
DispNum_Y:
        ld      A, D
        pop     DE
        cp      10
        jr      c, DispNum_Y_
        add     A, 'A'-'0'-10
DispNum_Y_:
        add     A, '0'
        ld      (DE), A
        inc     DE
        push    DE
        ld      D, 0
        jr      SearchBlackY_
SearchBlackY_1:
        inc     D
        .db     $DD \ ld H, B
        .db     $DD \ dec H
        jr      z, DispNum_Y
        cp      E
        jr      z, SearchBlackY_
        ld      D, 1
SearchBlackY_:
        ld      E, (HL)
        push    DE
        ld      A, (Size)
        and     A
        ld      DE, 8
        jr      z, SearchBlackY_Add
        add     HL, DE
SearchBlackY_Add:
        add     HL, DE
        pop     DE
        djnz    SearchBlackY
        pop     DE
        sub     A
        ld      (DE), A
        pop     BC
        call    DispVertInfo
        ret

DispVertInfo:
        push    BC
        sla     B
        ld      A, (Size)
        and     A
        jr      nz, DispVertInfo_1
        sla     B
DispVertInfo_1:
        ld      A, B
        add     A, 58
        cp      92
        jr      c, DispVertInfo_
        ld      A, 92
DispVertInfo_:
        ld      (CURSOR_X), A
        ld      B, A
        sub     A
        ld      (CURSOR_Y), A
        ld      HL, InfoTxtV
DispVertLoop:
        ld      A, (HL)
        inc     HL
        and     A
        jr      z, DispVertDone
        ROM_CALL(M_CHARPUT)
        ld      A, (CURSOR_Y)
        add     A, 6
        ld      (CURSOR_Y), A
        cp      31
        jr      c, DispVertLoop_
        sub     A
        ld      (CURSOR_Y), A
        ld      A, B
        add     A, 5
        ld      B, A
DispVertLoop_:
        ld      A, B
        ld      (CURSOR_X), A
        jr      DispVertLoop
DispVertDone:
        pop     BC
        ret

DoneStr:
     ;012345678901234567890
 .db "  You finished  "
 .db " all the levels "
 .db "  in this game. "
 .db "    Thank you   "
 .db "   for playing! ",0
GameOverStr:
 .db "-==*You Lose*==-",0
StageDone:
 .db "Press ENTER!",0
TimeUpStr:
 .db "Time",39,"s up!",0
MissStr:
 .db "Miss!",0
Title:
 .db "Picross v1.3 by:",0
ByStr:
 .db "Andreas Ess",0
 .db "Sam Heald",0
ScoreStr:
 .db "Time:",0
LivesStr:
 .db "Lives:",0
PauseTxt:
 .db "Score:",0
 .db "Best:",0
lev:
 .db "Level:",0
BestScore:
 .dw 0
BestLevel:
 .db 0

#include piclevel.h   ;Picross levels

Pointer:
 .db 4,4
 .db %11100000
 .db %11000000
 .db %10100000
 .db %00010000

Tile44:
 .db 4,4
 .db %11110000
 .db %11110000
 .db %11110000
 .db %11110000

Tile22:
 .db 2,2
 .db %11000000
 .db %11000000

DisplaySomeInfo:
	push bc
	ROM_CALL(CLEARLCD)
	  ld hl,Title
	  ld de,0
	  call DTX
        ld de,$0201
        call DTX
        ld de,$0302
        call DTX
        ld d,$00
        ld b,5
p_loop:
        inc e
        call DTX
        djnz p_loop
        
 

	ld      a, (Timer)
      ld l,a
      ld h,0
	ld      DE,$0803
	call    DCX

	ld      HL, (Score)
	ld      DE,$0805
	call    DCX

	ld      HL, (BestScore)
	ld      DE, $0806
      call    DCX	

	ld      DE, $0807
	ld      A, (Level)
	inc     A
	ld      L, A
	ld      H, 0
	call    DCX
      pop bc

	ld      DE, $0804
	ld      A, (Live)
	ld      L, A
	ld      H, 0
DCX:
 ld (CURSOR_ROW),de
 ROM_CALL(D_HL_DECI)
 ret


LevelSelectStr:
 .db "Select",0


DM_HL_DECI:
	push de
	push hl
 ld de,string+5
 xor a
 ld (de),a
RepUnp:
 dec de
 call UNPACK_HL
 add a,48
 ld (de),a
 djnz RepUnp
 ex de,hl
 ROM_CALL(D_ZM_STR)                 ;display string
	pop hl                        ;recall hl
	pop de                        ;recall de
	ret                           ;return to caller

LevelMatrix = APD_BUF
UserMatrix = APD_BUF+300


SHOW_GRAPH:
         push bc
         call     DISP_DELAY
         ld       a,7
         out      (16),a

         ld       c,17
         ld       a,$80
         ld       hl,GRAPH_MEM+1
loop_display:
         call     DISP_DELAY
         out      (16),a
         inc      a
         ld       e,a
         ld       a,$20
         call     DISP_DELAY
         out      (16),a
         ld       a,e

         call     DISP_DELAY \ outi
         call     DISP_DELAY \ outi
         call     DISP_DELAY \ outi
         call     DISP_DELAY \ outi
         call     DISP_DELAY \ outi
         call     DISP_DELAY \ outi
         call     DISP_DELAY \ outi
         call     DISP_DELAY \ outi
         call     DISP_DELAY \ outi
         call     DISP_DELAY \ outi
         call     DISP_DELAY \ outi
         call     DISP_DELAY \ outi

         cp       $c0
         jr       nz,loop_display
         pop bc
         ret
Delay:
 push bc
 ld b,5
HaltLoop:
      ei
	halt
	djnz	HaltLoop
      pop bc
      ret
DTX:
 ld (CURSOR_ROW),de
 ROM_CALL(D_ZT_STR)
 ret

DMX:
 ld (CURSOR_X),de
 ROM_CALL(D_ZM_STR)
 ret

DoTimer:
      ld hl,t_delay
      dec (hl)
      ret nz
 push bc
      ld a,100
      ld (t_delay),a
	ld      hl,Timer
      dec (hl)
      call DispTime
 pop bc
      ld a,(Timer)
      or a
      jr      z, LoseLife2
      ret

LoseLife2:
 inc sp
 inc sp
 jp LoseLife

 .db 0

.end

